﻿/* Мною, Константином aka KilkennyCat, 05 июля 2020 года создано сиё 
 * на основе "Описание формата TopoR PCB версия 1.2.0 Апрель 2017 г.".
 * k@kilkennycat.pro
 * http://kilkennycat.ru  http://kilkennycat.pro
 */

using System.Collections.Generic;
using System.Xml.Serialization;
using System.Linq;

namespace TopoR_PCB_Classes
{
    /// <summary>
    /// Раздел «Правила».
    /// </summary>
    /// <remarks>! Порядок следования правил в каждой секции определяет приоритет правил. Чем выше приоритет у правила, тем ниже оно описано.</remarks>
    public class Rules
    {
        /// <summary>
        /// Описание правила ширины проводников.
        /// </summary>
        public class WidthOfWires
        {
            /// <summary>
            /// Флаг применения правила.
            /// </summary>
            [XmlAttribute("enabled")]
            public Bool _enabled;
            [XmlIgnore]
            public bool _enabledSpecified { get { return _enabled != Bool.off; } }

            /// <summary>
            /// Параметр правила ширины проводников: минимальная ширина проводника.
            /// </summary>
            [XmlAttribute("widthMin", DataType = "float")]
            public float _widthMin;

            /// <summary>
            /// Параметр правила ширины проводников: номинальная ширина проводника.
            /// </summary>
            [XmlAttribute("widthNom", DataType = "float")]
            public float _widthNom;

            /// <summary>
            /// Ссылка на слои. См. также _LayersRefs
            /// </summary>
            /// <remarks>! При null необходимо смотреть _LayersRefs - там описан список ссылок типа LayerRef. </remarks>
            [
                 XmlElement("AllLayers", typeof(AllLayers)),
                 XmlElement("AllLayersInner", typeof(AllLayersInner)),
                 XmlElement("AllLayersInnerSignal", typeof(AllLayersInnerSignal)),
                 XmlElement("AllLayersSignal", typeof(AllLayersSignal)),
                 XmlElement("AllLayersOuter", typeof(AllLayersOuter)),
                 XmlElement("LayerGroupRef", typeof(LayerGroupRef))
            ]
            public object _LayersRef;
            /// <summary>
            /// Ссылка на слои. См. также _LayersRef
            /// </summary>
            /// <remarks>! При null необходимо смотреть _LayersRef - там описаны ссылки остальных типов. </remarks>
            [XmlElement("LayerRef")]
            public List<LayerRef> _LayersRefs;
            public bool ShouldSerialize_LayersRefs()
            {
                return _LayersRefs?.Count > 0;
            }
            /// <summary>
            /// Объекты воздействия правила.
            /// </summary>
            [XmlArray("ObjectsAffected")]
            [
                XmlArrayItem("NetRef", typeof(NetRef)),
                XmlArrayItem("NetGroupRef", typeof(NetGroupRef)),
                XmlArrayItem("AllNets", typeof(AllNets))
            ]
            public List<object> _ObjectsAffected;
            public bool ShouldSerialize_ObjectsAffected()
            {
                return _ObjectsAffected?.Count > 0;
            }
        }

        /// <summary>
        /// Описание правила зазоров между цепями.
        /// </summary>
        public class ClearanceNetToNet
        {
            /// <summary>
            /// Флаг применения правила.
            /// </summary>
            [XmlAttribute("enabled")]
            public Bool _enabled;
            [XmlIgnore]
            public bool _enabledSpecified { get { return _enabled != Bool.off; } }

            /// <summary>
            /// Параметр правила зазоров между цепями: минимальный зазор.
            /// </summary>
            [XmlAttribute("clrnMin", DataType = "float")]
            public float _clrnMin;

            /// <summary>
            /// Параметр правила зазоров между цепями: номинальный зазор.
            /// </summary>
            [XmlAttribute("clrnNom", DataType = "float")]
            public float _clrnNom;

            /// <summary>
            /// Ссылка на слои. См. также _LayersRefs
            /// </summary>
            /// <remarks>! При null необходимо смотреть _LayersRefs - там описан список ссылок типа LayerRef. </remarks>
            [
                 XmlElement("AllLayers", typeof(AllLayers)),
                 XmlElement("AllLayersInner", typeof(AllLayersInner)),
                 XmlElement("AllLayersInnerSignal", typeof(AllLayersInnerSignal)),
                 XmlElement("AllLayersSignal", typeof(AllLayersSignal)),
                 XmlElement("AllLayersOuter", typeof(AllLayersOuter)),
                 XmlElement("LayerGroupRef", typeof(LayerGroupRef))
            ]
            public object _LayersRef;
            /// <summary>
            /// Ссылка на слои. См. также _LayersRef
            /// </summary>
            /// <remarks>! При null необходимо смотреть _LayersRef - там описаны ссылки остальных типов. </remarks>
            [XmlElement("LayerRef")]
            public List<LayerRef> _LayersRefs;
            public bool ShouldSerialize_LayersRefs()
            {
                return _LayersRefs?.Count > 0;
            }
            /// <summary>
            /// Объекты воздействия правила.
            /// </summary>
            [XmlArray("ObjectsAffected")]
            [
                XmlArrayItem("NetRef", typeof(NetRef)),
                XmlArrayItem("NetGroupRef", typeof(NetGroupRef)),
                XmlArrayItem("AllNets", typeof(AllNets)),

                XmlArrayItem("SignalRef", typeof(SignalRef)),
                XmlArrayItem("DiffSignalRef", typeof(DiffSignalRef)),
                XmlArrayItem("SignalGroupRef", typeof(SignalGroupRef))
            ]
            public List<object> _ObjectsAffected;
            public bool ShouldSerialize_ObjectsAffected()
            {
                return _ObjectsAffected?.Count > 0;
            }
        }

        /// <summary>
        /// Описание правила зазоров между компонентами.
        /// </summary>
        public class ClearanceCompToComp
        {
            /// <summary>
            /// Флаг применения правила.
            /// </summary>
            [XmlAttribute("enabled")]
            public Bool _enabled;
            [XmlIgnore]
            public bool _enabledSpecified { get { return _enabled != Bool.off; } }

            /// <summary>
            /// Параметр правила зазоров между цепями: минимальный зазор.
            /// </summary>
            [XmlAttribute("clrn", DataType = "float")]
            public float _clrn;

            /// <summary>
            /// Объекты воздействия правила.
            /// </summary>
            [XmlArray("ObjectsAffected")]
            [
                XmlArrayItem("ComponentRef", typeof(ComponentRef)),
                XmlArrayItem("CompGroupRef", typeof(CompGroupRef)),
                XmlArrayItem("AllComps", typeof(AllComps))
            ]
            public List<object> _ObjectsAffected;
            public bool ShouldSerialize_ObjectsAffected()
            {
                return _ObjectsAffected?.Count > 0;
            }
        }

        /// <summary>
        /// Описание зазоров до края платы.
        /// </summary>
        public class RulesClearancesToBoard
        {
            /// <summary>
            /// Устанавливает зазор от проводников до края платы.
            /// </summary>
            [XmlAttribute("wires", DataType = "float")]
            public float _clrn;

            /// <summary>
            /// Устанавливает зазор от компонентов до края платы.
            /// </summary>
            [XmlAttribute("comps", DataType = "float")]
            public float _comps;
        }

        /// <summary>
        /// Описание правила назначения цепям стеков переходных отверстий.
        /// </summary>
        public class ViastacksOfNets
        {
            /// <summary>
            /// Флаг применения правила.
            /// </summary>
            [XmlAttribute("enabled")]
            public Bool _enabled;
            [XmlIgnore]
            public bool _enabledSpecified { get { return _enabled != Bool.off; } }

            /// <summary>
            /// Объекты воздействия правила.
            /// </summary>
            [XmlArray("ObjectsAffected")]
            [
                XmlArrayItem("NetRef", typeof(NetRef)),
                XmlArrayItem("NetGroupRef", typeof(NetGroupRef)),
                XmlArrayItem("AllNets", typeof(AllNets)),

                XmlArrayItem("SignalRef", typeof(SignalRef)),
                XmlArrayItem("DiffSignalRef", typeof(DiffSignalRef)),
                XmlArrayItem("SignalGroupRef", typeof(SignalGroupRef))
            ]
            public List<object> _ObjectsAffected;
            public bool ShouldSerialize_ObjectsAffected()
            {
                return _ObjectsAffected?.Count > 0;
            }
            /// <summary>
            /// Назначенные типы переходных отверстий.
            /// </summary>
            [XmlArray("Viastacks")]
            [
                XmlArrayItem("AllViastacks", typeof(AllViastacks)),
                XmlArrayItem("AllViastacksThrough", typeof(AllViastacksThrough)),
                XmlArrayItem("AllViastacksNotThrough", typeof(AllViastacksNotThrough)),

                XmlArrayItem("ViastackRef", typeof(ViastackRef))
            ]
            public List<object> _Viastacks;
            public bool ShouldSerialize_Viastacks()
            {
                return _Viastacks?.Count > 0;
            }
        }

        /// <summary>
        /// Описание правила назначения цепям опорных слоёв.
        /// </summary>
        public class PlaneLayerNets
        {
            /// <summary>
            /// Флаг применения правила.
            /// </summary>
            [XmlAttribute("enabled")]
            public Bool _enabled;
            [XmlIgnore]
            public bool _enabledSpecified { get { return _enabled != Bool.off; } }

            /// <summary>
            /// Ссылка на слои. См. также _LayersRefs
            /// </summary>
            /// <remarks>! При null необходимо смотреть _LayersRefs - там описан список ссылок типа LayerRef. </remarks>
            [
                 XmlElement("AllLayers", typeof(AllLayers)),
                 XmlElement("AllLayersInner", typeof(AllLayersInner)),
                 XmlElement("AllLayersInnerSignal", typeof(AllLayersInnerSignal)),
                 XmlElement("AllLayersSignal", typeof(AllLayersSignal)),
                 XmlElement("AllLayersOuter", typeof(AllLayersOuter)),
                 XmlElement("LayerGroupRef", typeof(LayerGroupRef))
            ]
            public object _LayersRef;
            /// <summary>
            /// Ссылка на слои. См. также _LayersRef
            /// </summary>
            /// <remarks>! При null необходимо смотреть _LayersRef - там описаны ссылки остальных типов. </remarks>
            [XmlElement("LayerRef")]
            public List<LayerRef> _LayersRefs;
            public bool ShouldSerialize_LayersRefs()
            {
                return _LayersRefs?.Count > 0;
            }
            /// <summary>
            /// Объекты воздействия правила.
            /// </summary>
            [XmlArray("ObjectsAffected")]
            [XmlArrayItem("NetRef")]
            public List<NetRef> _ObjectsAffected;
            public bool ShouldSerialize_ObjectsAffected()
            {
                return _ObjectsAffected?.Count > 0;
            }
        }

        /// <summary>
        /// Описание правила назначения цепям сигнальных слоёв.
        /// </summary>
        public class SignalLayerNets
        {
            /// <summary>
            /// Флаг применения правила.
            /// </summary>
            [XmlAttribute("enabled")]
            public Bool _enabled;
            [XmlIgnore]
            public bool _enabledSpecified { get { return _enabled != Bool.off; } }

            /// <summary>
            /// Ссылка на слои. См. также _LayersRefs
            /// </summary>
            /// <remarks>! При null необходимо смотреть _LayersRefs - там описан список ссылок типа LayerRef. </remarks>
            [
                 XmlElement("AllLayers", typeof(AllLayers)),
                 XmlElement("AllLayersInner", typeof(AllLayersInner)),
                 XmlElement("AllLayersInnerSignal", typeof(AllLayersInnerSignal)),
                 XmlElement("AllLayersSignal", typeof(AllLayersSignal)),
                 XmlElement("AllLayersOuter", typeof(AllLayersOuter)),
                 XmlElement("LayerGroupRef", typeof(LayerGroupRef))
            ]
            public object _LayersRef;
            /// <summary>
            /// Ссылка на слои. См. также _LayersRef
            /// </summary>
            /// <remarks>! При null необходимо смотреть _LayersRef - там описаны ссылки остальных типов. </remarks>
            [XmlElement("LayerRef")]
            public List<LayerRef> _LayersRefs;
            public bool ShouldSerialize_LayersRefs()
            {
                return _LayersRefs?.Count > 0;
            }
            /// <summary>
            /// Объекты воздействия правила.
            /// </summary>
            [XmlArray("ObjectsAffected")]
            [
                XmlArrayItem("NetRef", typeof(NetRef)),
                XmlArrayItem("NetGroupRef", typeof(NetGroupRef))
            ]
            public List<object> _ObjectsAffected;
            public bool ShouldSerialize_ObjectsAffected()
            {
                return _ObjectsAffected?.Count > 0;
            }
        }

        /// <summary>
        /// Свойства цепи.
        /// </summary>
        public class NetProperty
        {
            /// <summary>
            /// Свойство цепи: гибкая фиксация.
            /// </summary>
            [XmlAttribute("flexfix")]
            public Bool _flexfix;
            [XmlIgnore]
            public bool _flexfixSpecified { get { return _flexfix != Bool.off; } }

            /// <summary>
            /// Свойство цепи: флаг трассировки для автоматического трассировщика.
            /// </summary>
            [XmlAttribute("route")]
            public Bool _route;
            [XmlIgnore]
            public bool _routeSpecified { get { return _route != Bool.off; } }

            /// <summary>
            /// Ссылка на цепь.
            /// </summary>
            [XmlElement("NetRef")]
            public List<NetRef> _NetRefs;
            public bool ShouldSerialize_NetRefs()
            {
                return _NetRefs?.Count > 0;
            }
        }

        /// <summary>
        /// Настройки подключения к углам прямоугольных контактных площадок.
        /// </summary>
        public class PadConnectSettings
        {
            /// <summary>
            /// Настройка подключения к углам прямоугольных контактных площадок: режим подключения.
            /// </summary>
            [XmlAttribute("mode")]
            public mode_PadConnectSettings _mode;

            /// <summary>
            /// Ссылки на стеки контактных площадок.
            /// </summary>
            [XmlElement("PadstackRef")]
            public List<PadstackRef> _PadstackRefs;
            public bool ShouldSerialize_PadstackRefs()
            {
                return _PadstackRefs?.Count > 0;
            }
            /// <summary>
            /// Ссылки на контакты.
            /// </summary>
            [XmlElement("PinRef")]
            public List<PinRef> _PinRefs;
            public bool ShouldSerialize_PinRefs()
            {
                return _PinRefs?.Count > 0;
            }
            /// <summary>
            /// Ссылки на выводы посадочных мест.
            /// </summary>
            [XmlElement("PadRef")]
            public List<PadRef> _PadRefs;
            public bool ShouldSerialize_PadRefs()
            {
                return _PadRefs?.Count > 0;
            }
        }

        /// <summary>
        /// Версия раздела.
        /// </summary>
        [XmlAttribute("version")]
        public string _version;

        /// <summary>
        /// Правила ширин проводников.
        /// </summary>
        [XmlArray("RulesWidthOfWires")]
        [XmlArrayItem("WidthOfWires")]
        public List<WidthOfWires> _RulesWidthOfWires;
        public bool ShouldSerialize_RulesWidthOfWires()
        {
            return _RulesWidthOfWires?.Count > 0;
        }
        /// <summary>
        /// Правила зазоров между цепями.
        /// </summary>
        [XmlArray("RulesClearancesNetToNet")]
        [XmlArrayItem("ClearanceNetToNet")]
        public List<ClearanceNetToNet> _RulesClearancesNetToNet;
        public bool ShouldSerialize_RulesClearancesNetToNet()
        {
            return _RulesClearancesNetToNet?.Count > 0;
        }
        /// <summary>
        /// Правила зазоров между компонентами.
        /// </summary>
        [XmlArray("RulesClearancesCompToComp")]
        [XmlArrayItem("ClearanceCompToComp")]
        public List<ClearanceCompToComp> _RulesClearancesCompToComp;
        public bool ShouldSerialize_RulesClearancesCompToComp()
        {
            return _RulesClearancesCompToComp?.Count > 0;
        }
        /// <summary>
        /// Правило зазоров до края платы.
        /// </summary>
        [XmlElement("RulesClearancesToBoard")]
        public RulesClearancesToBoard _RulesClearancesToBoard;

        /// <summary>
        /// Правила назначения цепям стеков переходных отверстий.
        /// </summary>
        [XmlArray("RulesViastacksOfNets")]
        [XmlArrayItem("ViastacksOfNets")]
        public List<ViastacksOfNets> _RulesViastacksOfNets;
        public bool ShouldSerialize_RulesViastacksOfNets()
        {
            return _RulesViastacksOfNets?.Count > 0;
        }
        /// <summary>
        /// Правила назначения цепям опорных слоёв.
        /// </summary>
        [XmlArray("RulesPlaneLayersNets")]
        [XmlArrayItem("PlaneLayerNets")]
        public List<PlaneLayerNets> _RulesPlaneLayersNets;
        public bool ShouldSerialize_RulesPlaneLayersNets()
        {
            return _RulesPlaneLayersNets?.Count > 0;
        }
        /// <summary>
        /// Правила назначения цепям сигнальных слоёв.
        /// </summary>
        [XmlArray("RulesSignalLayersNets")]
        [XmlArrayItem("SignalLayerNets")]
        public List<SignalLayerNets> _RulesSignalLayersNets;
        public bool ShouldSerialize_RulesSignalLayersNets()
        {
            return _RulesSignalLayersNets?.Count > 0;
        }
        /// <summary>
        /// Свойства цепей
        /// </summary>
        [XmlArray("NetProperties")]
        [XmlArrayItem("NetProperty")]
        public List<NetProperty> _NetProperties;
        public bool ShouldSerialize_NetProperties()
        {
            return _NetProperties?.Count > 0;
        }
        /// <summary>
        /// Настройки подключения к углам прямоугольных контактных площадок.
        /// </summary>
        [XmlElement("PadConnectSettings")]
        public PadConnectSettings _PadConnectSettings;


        /*****************************************************************
         * Здесь находятся функции для работы с элементами класса Rules. *
         * Они не являются частью формата TopoR PCB.                     *
         * ***************************************************************/

        /// <summary>
        /// Переименование ссылок на компонент, если его имя изменилось
        /// </summary>
        /// <param name="oldname">старое имя компонента</param>
        /// <param name="newname">новое имя компонента</param>
        public void Rename_compName(string oldname, string newname)
        {
            foreach (var a in _PadConnectSettings?._PadRefs?.Where(aa => aa._compName == oldname))
            {
                a._compName = newname;
            }
            foreach (var a in _PadConnectSettings?._PinRefs?.Where(aa => aa._compName == oldname))
            {
                a._compName = newname;
            }

            foreach (var a in _RulesClearancesCompToComp?.Where(aa => aa._ObjectsAffected != null))
            {
                foreach (var b in a._ObjectsAffected.OfType<ComponentRef>().Where(bb => bb._ReferenceName == oldname))
                {
                    b._ReferenceName = newname;
                }
            }

        /****************************************************************/

        }
    }
}
